﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerControl : MonoBehaviour
{
    //sound
    [SerializeField] private PlayerSound ps;

    // Movement
    public float MaxSpeed = 1500;
    public float Speed = 750;

    // Jump
    public float JumpBoost = 10f;
    public float JumpAccel = 2500f;
    public float MinJumpTime = .025f;
    public float MaxJumpTime = .15f;

    // Throw
    public Vector2 ThrowSpeedRange = new Vector2(350, 800);
    public float ThrowChargeTime = 2;
    public float ThrowPercentExponential = .5f;


    // Runtime
    public bool JumpTrigger = false;
    public bool JumpPressed = false;
    public bool Jumping = false;
    JumpMonitor JumpMonitor = new JumpMonitor();
    Timer MinJump = new Timer();
    Timer MaxJump = new Timer();
    Timer ChargeTimer = new Timer();
    public bool Charging = false;
    public bool WantStartCharge = false;
    public bool WantFinishCharge = false;
    public Vector2 Aim = new Vector2(0, 0);

    protected Brick Held = null;

    protected Rigidbody2D _Rigid = null;
    public Rigidbody2D Rigid
    {
        get
        {
            if (_Rigid == null)
                _Rigid = GetComponent<Rigidbody2D>();

            return _Rigid;
        }
    }

    protected Animator _Animator = null;
    public Animator Animator
    {
        get
        {
            if (_Animator == null)
                _Animator = GetComponentInChildren<Animator>();

            return _Animator;
        }
    }

    protected PlayerSensor _Sensor = null;
    public PlayerSensor Sensor
    {
        get
        {
            if (_Sensor == null)
                _Sensor = GetComponentInChildren<PlayerSensor>();

            return _Sensor;
        }
    }

    protected ChargeBar _ChargeBar = null;
    public ChargeBar ChargeBar
    {
        get
        {
            if (_ChargeBar == null)
                _ChargeBar = GetComponentInChildren<ChargeBar>();

            return _ChargeBar;
        }
    }

    protected HardPoint _Muzzle = null;
    public HardPoint Muzzle
    {
        get
        {
            if (_Muzzle == null)
                _Muzzle = GetComponentInChildren<HardPoint>();

            return _Muzzle;
        }
    }

    public Vector2 HoldPosition
    {
        get
        {
            return Muzzle.transform.position;
        }
    }

    public Vector2 AimVector
    {
        get
        {
            Vector3 source_pos = HoldPosition;
            Vector2 between = new Vector2(Aim.x - source_pos.x, Aim.y - source_pos.y);

            return between.normalized;
        }
    }

    public Vector2 MuzzleOffset
    {
        get
        {
            return Muzzle.transform.position - transform.position;
        }
    }

    void Start()
    {
        if (ps == null)
            Debug.LogError("ps is null in PlayerController");

        ChargeTimer.Reset(ThrowChargeTime);
    }

    void Update()
    {
        bool want_jump = JumpMonitor.CheckJump("Vertical");
        if (want_jump)
        {
            if (!Jumping && Sensor.IsOnGround)
            {
                JumpTrigger = true;
            }
        }

        Vector3 cursor = Camera.main.ScreenToWorldPoint(Input.mousePosition);

        Aim = new Vector2(cursor.x, cursor.y);

        if (Input.GetMouseButtonUp(0))
            WantFinishCharge = true;

        if (Input.GetMouseButton(0))
            WantStartCharge = true;

        JumpPressed = want_jump;

        Animator.SetFloat("Speed", Mathf.Abs(Rigid.velocity.x));
        Animator.SetFloat("Throttle", Input.GetAxisRaw("Horizontal"));
        Animator.SetBool("Hold", Held != null);
        Animator.SetBool("InAir", !Sensor.IsOnGround);
    }

    private void FixedUpdate()
    {
        HandleMovement();
        HandleBrick();
        HandleCharge();
    }

    public void HandleMovement()
    {
        {   // Horizontal Movement
            float throttle = Input.GetAxisRaw("Horizontal");
            float step = Time.fixedDeltaTime;

            MoveUtil.ClampedHorizontalMovement(Rigid, Speed, MaxSpeed, throttle, step);
        }

        {   // Jump
            if (!Jumping && JumpTrigger)
                Jump();

            if (Jumping)
                TickJump();
        }
    }

    public void Jump()
    {
        JumpTrigger = false;
        Jumping = true;

        MinJump.Reset(MinJumpTime);
        MaxJump.Reset(MaxJumpTime);

        ps.PlayJumpSound();
        MoveUtil.BasicJump(Rigid, JumpBoost);
    }

    public void TickJump()
    {
        MinJump.Tick(Time.fixedDeltaTime);
        MaxJump.Tick(Time.fixedDeltaTime);

        if (!MinJump.Done || JumpPressed)
        {
            Rigid.AddForce(new Vector2(0, Time.fixedDeltaTime * MaxJump.PercentStep * JumpAccel));
        }

        if (MinJump.Done && !JumpPressed && Sensor.IsOnGround)
        {
            Jumping = false;
            ps.PlayLandSound();
        }
    }

    public void HandleBrick()
    {
        if (Held == null)
        {
            Brick brick = Sensor.GetBrick();

            if (brick != null)
            {
                Held = brick;
                brick.Grab(this);

            }
        }
    }

    public void StartCharge()
    {
        Charging = true;
        ChargeTimer.Reset(ThrowChargeTime);
        ps.PlayChargeSound();
    }

    public void HandleCharge()
    {
        if (Held != null)
        {
            if (!Charging && WantStartCharge)
                StartCharge();

            if (Charging)
                ChargeTimer.Tick(Time.fixedDeltaTime);

            if (Charging && WantFinishCharge)
            {
                Charging = false;
                ThrowBrick();
                ps.StopSound();
            }
        }

        ChargeBar.SetPercent(ChargeTimer.PercentDone);
        ChargeBar.SetLook(AimVector);

        WantStartCharge = false;
        WantFinishCharge = false;
    }

    public void ThrowBrick()
    {
        if (Held != null)
        {
            float percent = ChargeTimer.PercentDone;
            float amount = Mathf.Lerp(ThrowSpeedRange.x, ThrowSpeedRange.y, Mathf.Lerp(percent, percent*percent, ThrowPercentExponential));


            if (ChargeTimer.PercentDone >= (1 - 1e-3f))
            {
                Animator.SetTrigger("Yeet");
            }

            Held.Throw(AimVector * amount);
            Held = null;
            ChargeTimer.Reset();

        }
    }
}
